program Md5Check;

(*******************************************************************************
*                                                                              *
* Author    :  Angus Johnson                                                   *
* Date      :  9 March 2012                                                    *
* Website   :  http://www.angusj.com                                           *
* Copyright :  Angus Johnson 2010-2012                                         *
*                                                                              *
* License:                                                                     *
* Use, modification & distribution is subject to Boost Software License Ver 1. *
* http://www.boost.org/LICENSE_1_0.txt                                         *
*                                                                              *
*******************************************************************************)

uses
  Windows,
  messages,
  CommDlg,
  SysUtils,
  CommCtrl,
  ShellApi,
  MD5 in 'MD5.pas';

{$R md5.res}

const
  version = 'ver 3.0';

  //dialog items...
  idCalculate = 3;
  idResult = 4;
  idMatch = 5;
  idResultIcon = 6;
  idClose = 7;
  idBrowse = 8;
  idFilename = 9;
  idProgressbar = 10;
  idMessage = 11;
  idVersion = 12;
  idIconImage = 13;

type

  THelper = class
  public
    cancelled: boolean;
    procedure OnProgress(sender: TObject; percent: integer; var cancel: boolean);
  end;

  {this structure definition changes between delphi versions hence
  to enable consistency between compilers...}
  TOpenFilename = record
    lStructSize: DWORD;
    hWndOwner: HWND;
    hInstance: HINST;
    lpstrFilter: PChar;
    lpstrCustomFilter: PChar;
    nMaxCustFilter: DWORD;
    nFilterIndex: DWORD;
    lpstrFile: PChar;
    nMaxFile: DWORD;
    lpstrFileTitle: PChar;
    nMaxFileTitle: DWORD;
    lpstrInitialDir: PChar;
    lpstrTitle: PChar;
    Flags: DWORD;
    nFileOffset: Word;
    nFileExtension: Word;
    lpstrDefExt: PChar;
    lCustData: LPARAM;
    lpfnHook: function(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT stdcall;
    lpTemplateName: PChar;
    pvReserved: Pointer;
    dwReserved: DWORD;
    FlagsEx: DWORD;
  end;

  function GetOpenFileName(var OpenFile: TOpenFilename): Bool; stdcall;
    external 'comdlg32.dll'  name
{$IFDEF UNICODE}
                                   'GetOpenFileNameW';
{$ELSE}
                                   'GetOpenFileNameA';
{$ENDIF}

var
  dlg: THandle;
  boldFont: HFont;
  icon16, icon32, iconTick, iconCross, iconImage: HIcon;

  YellowBrush: HBrush;
  LimeBrush: HBrush;
  BtnFaceBrush: HBrush;

  osVersionInfo: TOSVersionInfo;

//---------------------------------------------------------------------
//---------------------------------------------------------------------

function OpenFileDialog(Owner: THandle; const Caption, Filter,
  InitialDir: string; var Filename: string; MultiSelect: boolean): boolean;
var
  ofn: TOpenFilename;
  filepath: array [0..1024] of char;
  i: integer;
  newfilter: string;
begin
  strPCopy(filepath,Filename); //nb: ofn.lpstrFile must be initialized (or #0)

  //nb: assumes only a single filter here...
  i := pos('|',Filter);
  if i > 0 then
  begin
    newfilter  := copy(Filter,1,i-1)+#0+copy(filter,i+1,255)+#0#0;
  end;
  if newfilter = '' then
    newfilter  := 'All Files (*.*)'+#0'*.*'#0#0;

  fillchar(ofn, sizeof(ofn), 0);
  //prior to win2000 & WinME the last 3 fields of TOpenFilename aren't supported
  if ((osVersionInfo.dwMajorVersion > 4) and
      (osVersionInfo.dwPlatformId = VER_PLATFORM_WIN32_NT)) {win2000 or winXP} or
      ((osVersionInfo.dwMajorVersion > 3) and
      (osVersionInfo.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS) and
      (osVersionInfo.dwMinorVersion >= 90)) {winME} then
    ofn.lStructSize := sizeof(TOpenFilename) else
    ofn.lStructSize := sizeof(TOpenFilename) -12;

  ofn.hWndOwner := Owner;
  ofn.lpstrFilter := pchar(newfilter);
  ofn.nFilterIndex := 1;
  ofn.lpstrFile := pchar(@filepath[0]);
  ofn.nMaxFile := MAX_PATH;
  ofn.lpstrFileTitle := nil;

  ofn.lpstrInitialDir := pchar(InitialDir);
  ofn.lpstrTitle := pchar(caption);
  ofn.Flags := OFN_FILEMUSTEXIST or OFN_PATHMUSTEXIST or OFN_HIDEREADONLY;
  if MultiSelect then
    ofn.Flags := ofn.Flags or OFN_ALLOWMULTISELECT or OFN_EXPLORER;
  result := GetOpenFileName(ofn);
  if not result then exit;
  //if MultiSelect then filepath can have 2 general forms:
  //  1. 'path&name'#0#0 or
  //  2. 'path'#0'name1'#0'name2'#0' ..nameN'#0#0
  if MultiSelect then
  begin
    //here, we're just interested in the first filename, so ...
    Filename := pchar(@filepath[strlen(filepath)+1]);
    if Filename = '' then
      Filename := filepath else
      Filename := format('%s\%s',[filepath, Filename]);
  end
  else
    Filename := filepath;
end;
//--------------------------------------------------------------------------

procedure THelper.OnProgress(sender: TObject; percent: integer; var cancel: boolean);
var
  msg: TMsg;
begin
  //if percent >= 100 then percent := 0;
  SendDlgItemMessage(dlg, idProgressbar, PBM_SETPOS, percent, 0);
  //process messages to avoid 'non-responding' with very large files ...
  while PeekMessage(Msg, 0, 0, 0, PM_REMOVE) do
  begin
    if Msg.Message = WM_QUIT then
    begin
      cancel := true;
      cancelled := true; //save result to helper too
      exit;
    end;
    IsDialogMessage(dlg, msg);
  end;
end;
//--------------------------------------------------------------------------

procedure CalcMD5Sum;
var
  oldCursor: HCursor;
  md5: TMd5;
  s: string;
  helper: THelper;
begin
  EnableWindow(GetDlgItem(dlg,idCalculate), false);
   oldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
   helper := THelper.create;
   md5 := TMd5.Create;
   try
     helper.cancelled := false;
     s := '&Cancel';
     sendDlgItemMessage(dlg, idClose, WM_SETTEXT, 0, longint(pchar(s)));

     UpdateWindow(dlg);
     setlength(s, 1024);
     sendDlgItemMessage(dlg, idFilename, WM_GETTEXT, 1024, longint(@s[1]));
     md5.OnProgress := helper.OnProgress;
     s := md5.GetHash(s);
     if (s <> '') then //success!
     begin
       sendDlgItemMessage(dlg, idResult, WM_SETTEXT, 0, longint(pchar(s)));
     end
     else if helper.cancelled then
     begin
       SendDlgItemMessage(dlg, idProgressbar, PBM_SETPOS, 0, 0);
       EnableWindow(GetDlgItem(dlg,idCalculate), true);
     end else
     begin
       s := 'Error processing MD5 checksum!';
       sendDlgItemMessage(dlg, idMessage, WM_SETTEXT, 0, longint(pchar(s)));
     end;
   finally
     helper.free;
     md5.free;
     s := '&Close';
     sendDlgItemMessage(dlg, idClose, WM_SETTEXT, 0, longint(pchar(s)));
     SetCursor(oldCursor);
   end;
end;
//--------------------------------------------------------------------------

procedure DoCompare(const s1, s2: string);
var
  buff: array[0..254] of char;
begin
  if CompareText(trim(s1), trim(s2)) = 0 then
    StrPCopy(buff,'MD5 checksums match.') else
    StrPCopy(buff,'MD5 checksums DON''T match.');
  sendDlgItemMessage(dlg, idMessage, WM_SETTEXT, 0, longint(@buff[0]));
end;
//--------------------------------------------------------------------------


function Main(Dialog: HWnd; AMessage, WParam: LongInt; LParam: Longint): Bool; stdcall;
const
  bufflen = 1024;
var
  buff: array[0..bufflen -1] of char;
  s1, s2, fn: string;
  hdl: THandle;
begin
  result := true;
  case AMessage of
    WM_INITDIALOG:
      begin
        //set the application icons ...
        SetClassLong( Dialog, GCL_HICON,
          LoadImage(hInstance,'MAINICON', IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR));
        SendMessage(Dialog, WM_SETICON, ICON_BIG, icon32);
        SendMessage(Dialog, WM_SETICON, ICON_SMALL, icon16);
        SendDlgItemMessage(Dialog, idResult, WM_SETFONT, boldFont, 0);
        SendDlgItemMessage(Dialog, idMatch, WM_SETFONT, boldFont, 0);
        SendDlgItemMessage(Dialog, idMessage, WM_SETFONT, boldFont, 0);
        sendDlgItemMessage(Dialog, idVersion, WM_SETTEXT, 0, longint(PChar(version)));
        sendDlgItemMessage(Dialog, idIconImage, STM_SETICON, iconImage, 0);
        //SendDlgItemMessage(dlg, idProgressbar, PBM_SETRANGE, 0, MAKELPARAM(0, 100));
      end;
    WM_COMMAND:
      case LoWord(WParam) of
        IDCANCEL, idClose: PostQuitMessage(0); //modeless dialogs only
        idFilename:
          if HiWord(WParam) = EN_CHANGE then
          begin
            SendDlgItemMessage(dlg, idProgressbar, PBM_SETPOS, 0, 0);
            //enable/disable Calculate button based on Filename ...
            sendDlgItemMessage(dlg, idFilename, WM_GETTEXT, bufflen,longint(@buff[0]));
            EnableWindow(GetDlgItem(dlg,idCalculate), fileExists(buff));
            //clear Result and Message windows ...
            buff[0] := #0;
            sendDlgItemMessage(dlg, idResult, WM_SETTEXT, 0, longint(@buff[0]));
            sendDlgItemMessage(dlg, idMessage, WM_SETTEXT, 0, longint(@buff[0]));

          end;
        idCalculate: CalcMD5Sum;
        idBrowse:
          if (HIWORD(wParam) = BN_CLICKED) and
            OpenFileDialog(dlg,'Select File ...', 'All Files (*.*)|*.*','',fn, false) then
          begin
            sendDlgItemMessage(dlg, idFilename, WM_SETTEXT, 0, longint(pchar(fn)));
            SetFocus(GetDlgItem(dlg,idMatch));
            CalcMD5Sum;
          end;
        idResult, idMatch:
          if HiWord(WParam) = EN_CHANGE then
          begin
            sendDlgItemMessage(dlg, idResult, WM_GETTEXT, bufflen,longint(@buff[0]));
            s1 := buff;
            sendDlgItemMessage(dlg, idMatch, WM_GETTEXT, bufflen,longint(@buff[0]));
            s2 := buff;
            if (s1 <> '') and (s2 <> '') then
              DoCompare(s1, s2)
            else
            begin
              buff[0] := #0;
              sendDlgItemMessage(dlg, idMessage, WM_SETTEXT, 0, longint(@buff[0]));
            end;
          end;
      end;
    WM_DROPFILES:
      begin
        DragQueryFile(WParam, 0, @buff, bufflen);
        sendDlgItemMessage(dlg, idFilename, WM_SETTEXT, 0, longint(@buff[0]));
        dragfinish(WParam);
        CalcMD5Sum;
      end;
    WM_CTLCOLORSTATIC:
      begin
        hdl := GetDlgItem(dlg, idMessage);
        if (cardinal(lparam) = hdl) then
        begin
          GetWindowText(hdl,buff,bufflen);
          if (buff = '') then
          begin
            SetBkColor(wParam,  GetSysColor(COLOR_BTNFACE and $000000FF));
            result := Bool(BtnFaceBrush);
          end
          else if buff = 'MD5 checksums match.' then
          begin
            SetBkColor(wParam, RGB(0,255,0));
            result := Bool(LimeBrush);
            SendMessage(GetDlgItem(dlg, idResultIcon), STM_SETICON, iconTick, 0);
          end
          else
          begin
            SetBkColor(wParam, RGB(255,255,0));
            result := Bool(YellowBrush);
            SendMessage(GetDlgItem(dlg, idResultIcon), STM_SETICON, iconCross, 0);
          end;
        end
        else result := false;
      end;
    else result := False; //ie not handled
  end;
end;

//--------------------------------------------------------------------------
// Main prog. entry point...
//--------------------------------------------------------------------------
var
  msg: TMsg;
  param1: string;
begin

  osVersionInfo.dwOSVersionInfoSize := sizeof(osVersionInfo);
  GetVersionEx(osVersionInfo);

  icon16 := LoadImage(hInstance,'MAINICON', IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
  icon32 := LoadImage(hInstance,'MAINICON', IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);
  iconImage := LoadImage(hInstance,'ZLOGO', IMAGE_ICON, 88, 32, LR_DEFAULTCOLOR);

  iconTick := LoadIcon(hInstance, 'ZTICK');
  iconCross := LoadIcon(hInstance, 'ZCROSS');

  //Create a bold font for the about dialog...
  boldFont := CreateFont(-11, 0, 0, 0, FW_EXTRABOLD, 0, 0, 0, DEFAULT_CHARSET,
                      OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
                      DEFAULT_PITCH or FF_DONTCARE, 'Arial');

  YellowBrush := CreateSolidBrush(RGB(255,255,0));
  LimeBrush := CreateSolidBrush(RGB(0,255,0));
  BtnFaceBrush := CreateSolidBrush(GetSysColor(COLOR_BTNFACE));

  dlg := CreateDialog( hInstance, 'MAIN', 0, @Main);

  if (paramcount > 0) then
  begin
    param1 := paramstr(1);
    sendDlgItemMessage(dlg, idFilename, WM_SETTEXT, 0, longint(pchar(param1)));
    if fileExists(param1) then
    begin
      EnableWindow(GetDlgItem(dlg,idCalculate), true);
      SetFocus(GetDlgItem(dlg,idCalculate));
    end
    else
      sendDlgItemMessage(dlg, idFilename, EM_SETSEL, 0, -1);
  end;

  while GetMessage(msg, 0, 0, 0) do
    IsDialogMessage(dlg, msg);

  //cleanup...
  DestroyIcon(Icon16);
  DestroyIcon(Icon32);
  DestroyIcon(IconTick);
  DestroyIcon(IconCross);
  DestroyIcon(IconImage);

  DeleteObject(YellowBrush);
  DeleteObject(LimeBrush);
  DeleteObject(BtnFaceBrush);

  DeleteObject(boldFont);
  DestroyWindow(dlg);
end.

